--[==[

	Copyright (c) 2019, S T Pain

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all
	copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
	SOFTWARE.
	
	-----------------------------------------------------------------------------
	FORAGER
	
	This addon is designed to record herb and ore location that the player finds
	
	Originally written by Silvessa (Darkmoon Faire, then later Argent Dawn)
	
]==]--

local name, Forager = ...

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- ADDON VARIABLES
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--|cffffcc00 - yellow header font

-- table for functions, can be moved to seperate file in time
Forager.Functions = {}

-- used to hold the icons displayed on the world map
Forager.MapNodeCache = {}

-- used to hold the icons displayed on the minimap
Forager.MapNodeCacheMinimap = {}

--spell id of mining and herbalism spells, used to check loot is from a node

-- used to determine if the player is tracking herbs or ore or nothing
Forager.TrackingIcons = { [133939] = 'Herb', [136025] = 'Ore' }

-- used to determine minimap position
Forager.MinimapScales = { [0] = 7.0, [1] = 9.0, [2] = 11.5, [3] = 14.0, [4] = 18.0, [5] = 27.0 }

-- use global variables rather than constantly create locals when getting positions
Forager.CurrentMapID = nil
Forager.CurrentMapPosition = nil
Forager.CurrentMapName = nil
Forager.WorldMapZoneDisplayed = nil
Forager.IsZoneDisplayedInZoneTable = false --current world map display, does the db have any entries for that zone


-- seems to be 0.0011 ish as a distance you can cast from node, this uses world map 0-1 axis values
Forager.GatherSpellRange = 0.0025 --this is roughly double the mining spell range HOWEVER this may need checking with time and adjusting, the idea being we can find a matching node even if player stands on opposites sides.

-- holds a boolean for last spell cast, true if it was a node gathering spell
Forager.IsForagingSpellCast = false

Forager.NodeUpdated = false
Forager.LootUpdated = false
Forager.LastLootSpell = nil; -- mining or herbs
Forager.SpellTarget = 'none'
Forager.ChestLooted = false

----------------------------------------------------------------------------
-- old variables no longer used, except distance checking via slash commands
----------------------------------------------------------------------------
Forager.OldCurrentMapID = nil;
Forager.OldCurrentMapPosition = nil;
Forager.OldCurrentMapName = nil;


--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- REGISTER EVENTS
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Forager.MainFrame = CreateFrame('FRAME', nil, UIParent)
Forager.MainFrame:RegisterEvent('UNIT_SPELLCAST_SENT')
Forager.MainFrame:RegisterEvent('CHAT_MSG_LOOT')
Forager.MainFrame:RegisterEvent('ADDON_LOADED')
Forager.MainFrame:RegisterEvent('CHAT_MSG_ADDON')
Forager.MainFrame:RegisterEvent('PLAYER_REGEN_DISABLED')
Forager.MainFrame:RegisterEvent('MINIMAP_UPDATE_TRACKING')

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- SLASH COMMANDS
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

-- mainly for testing and debug
SLASH_FORAGER1 = '/forager'
SlashCmdList['FORAGER'] = function(msg)
	if msg == 'pos a' then --print current position and distance to last position if exist
		Forager.CurrentMapID = C_Map.GetBestMapForUnit('player')
		Forager.OldCurrentMapPosition = C_Map.GetPlayerMapPosition(Forager.CurrentMapID, 'player')
		print('location', string.format('%.2f', Forager.OldCurrentMapPosition.x * 140), string.format('%.2f', Forager.OldCurrentMapPosition.y * 140), C_Map.GetMapInfo(Forager.CurrentMapID).name)		

	elseif msg == 'pos b' then
		Forager.CurrentMapID = C_Map.GetBestMapForUnit('player')
		Forager.CurrentMapPosition = C_Map.GetPlayerMapPosition(Forager.CurrentMapID, 'player')
		if Forager.OldCurrentMapPosition ~= nil then 
			local x_dis = (Forager.OldCurrentMapPosition.x - Forager.CurrentMapPosition.x)
			local y_dis = (Forager.OldCurrentMapPosition.y - Forager.CurrentMapPosition.y)	
			local dis = ((x_dis^2.0)+(y_dis^2.0))^0.5
			print('Distance', string.format('%.2f', dis * 140))
		end		
		
	elseif msg == 'purge' then
		StaticPopup_Show ("Forager_Purge")
		
	elseif msg == 'capture' then
		Forager.CaptureFrame:Show()
		Forager.CaptureFrame.SpellIDExist:SetText('')
		Forager.CaptureFrame.AddButton:SetEnabled(false)
		
	elseif msg == 'list-spells' then
		for k, spell in pairs(ForagerGlobalSettings.SpellIDs) do
			if spell then
				local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo(tonumber(spell))
				Forager.Functions.Print(tostring(name..' - '..spell), true)
			end
		end	
		
	elseif msg == 'remove-spell' then
	
		StaticPopup_Show ("Forager_RemoveSpell")

	end
end

StaticPopupDialogs["Forager_RemoveSpell"] = {
	text = "To remove a spell added via 'Capture' enter the spell id below, clicking print spells will cause this dialog to close.",
	button1 = "Ok",
	button2 = 'Cancel',
	button3 = 'Print Spells',
	OnAlt = function()
		for k, spell in pairs(ForagerGlobalSettings.SpellIDs) do
			if spell then
				local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo(tonumber(spell))
				Forager.Functions.Print(tostring(name..' - '..spell), true)
			end
		end	
	end,
	hasEditBox = true,
	EditBoxOnTextChanged = function (self)
		local exists = false
		for k, spell in pairs(ForagerGlobalSettings.SpellIDs) do
			if spell then
				if tonumber(spell) == tonumber(self:GetText()) then
					exists = true
				end
			end
		end
		if exists == true then
			self:GetParent().button1:Enable()
			local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo(tonumber(self:GetText()))
			self:GetParent().text:SetText(name)
		else
			self:GetParent().button1:Disable()
		end
	end,
	OnShow = function(self) self.editBox:SetText('Spell ID') self.button1:Disable() end,
	OnAccept = function(self) 
		for k, spell in pairs(ForagerGlobalSettings.SpellIDs) do
			if spell then
				if tonumber(spell) == tonumber(self.editBox:GetText()) then
					local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo(tonumber(self.editBox:GetText()))
					ForagerGlobalSettings.SpellIDs[k] = nil
					Forager.Functions.Print(tostring('removed '..name), true)
				end
			end
		end
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
}

StaticPopupDialogs["Forager_Purge"] = {
	text = "Are you sure you want to delete all nodes from the database?\n\nThis will trigger a UI reload!",
	button1 = "Ok",
	button2 = 'Cancel',
	OnAccept = function() ForagerGlobalSettings.NodesLootedDatabaseBackUp = ForagerGlobalSettings.NodesLootedDatabase, ForagerGlobalSettings.NodesLootedDatabase = {} Forager.Functions.Print('database deleted') ReloadUI() end,
	OnCancel = function() Forager.Functions.Print('database NOT deleted') end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = false,
	preferredIndex = 3,
}

StaticPopupDialogs["Forager_AddAbility"] = {
	text = "Are you sure you want to add the ability \n%s \nto the foraging database? \n\nThis will cause node data capture everytime the ability/spell is used!",
	button1 = "Ok",
	button2 = 'Cancel',
	OnAccept = function()
		table.insert(ForagerGlobalSettings.SpellIDs, Forager.CaptureSpellID)
		Forager.CaptureFrame.SpellIDExist:SetText('') 
		Forager.CaptureFrame:Hide()
		Forager.Functions.Print('added new foraging ability to database') end,
	OnCancel = function() Forager.Functions.Print('ability not added') Forager.CaptureFrame:Hide() end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = false,
	preferredIndex = 3,
}
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- OPTIONS INTERFACE PANEL
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

-- this can be converted into an xml file in fuure updates

--options interface
Forager.OptionsPanel = CreateFrame("Frame", "ForagerOptionsPanel", UIParent);
Forager.OptionsPanel.name = "Forager";
InterfaceOptions_AddCategory(Forager.OptionsPanel);

--add a title
Forager.OptionsPanel.Title = Forager.OptionsPanel:CreateFontString("ForagerOptionsPanelTitle", "OVERLAY", "GameFontNormal")
Forager.OptionsPanel.Title:SetPoint("TOPLEFT", 16, -16)
Forager.OptionsPanel.Title:SetFont("Fonts\\FRIZQT__.TTF", 18)
Forager.OptionsPanel.Title:SetText("Forager")

--about
Forager.OptionsPanel.About = Forager.OptionsPanel:CreateFontString('ForagerOptionsPanelAbout', 'OVERLAY', 'GameFontNormal')
Forager.OptionsPanel.About:SetPoint('TOPLEFT', 16, -60)
Forager.OptionsPanel.About:SetSize(580, 50)
Forager.OptionsPanel.About:SetFont("Fonts\\FRIZQT__.TTF", 11)
Forager.OptionsPanel.About:SetJustifyH('LEFT')
Forager.OptionsPanel.About:SetText('Forager is a simple addon based on the popular Gatherer addon. Forager will track what you discover in the open world and provide a simple reminder on your world map, just in case you forgot where you found that precious ore or herb.')
Forager.OptionsPanel.About:SetTextColor(1,1,1,1)

-- sharing options header
Forager.OptionsPanel.SharingOptionsHeader = Forager.OptionsPanel:CreateFontString('ForagerOptionsPanelSharingOptionsHeader', 'OVERLAY', 'GameFontNormal')
Forager.OptionsPanel.SharingOptionsHeader:SetPoint('TOPLEFT', 16, -120)
--Forager.OptionsPanel.SharingOptionsHeader:SetSize(580, 50)
Forager.OptionsPanel.SharingOptionsHeader:SetFont("Fonts\\FRIZQT__.TTF", 13)
Forager.OptionsPanel.SharingOptionsHeader:SetJustifyH('LEFT')
Forager.OptionsPanel.SharingOptionsHeader:SetText('General Settings')
--Forager.OptionsPanel.SharingOptionsHeader:SetTextColor(1,1,1,1)

-- guild share check box
Forager.OptionsPanel.GuildShareCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelGuildShareCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.GuildShareCheckBox:SetPoint('TOPLEFT', 20, -140)
ForagerOptionsPanelGuildShareCheckBoxText:SetText('|cffffffffShare finds with guild|r')
Forager.OptionsPanel.GuildShareCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].GuildShare = true
		else
			ForagerGlobalSettings[UnitGUID('player')].GuildShare = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].GuildShare = false
		else
			ForagerGlobalSettings[UnitGUID('player')].GuildShare = false
		end
	end
	Forager.Functions.Print('sharing locations with guilds: '..tostring(ForagerGlobalSettings[UnitGUID('player')].GuildShare))
end)
Forager.OptionsPanel.GuildShareCheckBox.tooltip = 'Share locations for foraged treasure?'

--print info
Forager.OptionsPanel.PrintForagerInfoCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelPrintForagerInfoCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.PrintForagerInfoCheckBox:SetPoint('TOPLEFT', 300, -140)
ForagerOptionsPanelPrintForagerInfoCheckBoxText:SetText('|cffffffffPrint Forager Information|r')
Forager.OptionsPanel.PrintForagerInfoCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo = true
		else
			ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo = false
		else
			ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo = false
		end
		Forager.HideMinimapPins()
	end
	Forager.Functions.Print('print information: '..tostring(ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo))
end)
Forager.OptionsPanel.PrintForagerInfoCheckBox.tooltip = 'Print information from forager.'





--world map options
Forager.OptionsPanel.WorldMapOptionsHeader = Forager.OptionsPanel:CreateFontString('ForagerOptionsPanelWorldMapOptionsHeader', 'OVERLAY', 'GameFontNormal')
Forager.OptionsPanel.WorldMapOptionsHeader:SetPoint('TOPLEFT', 16, -200)
--Forager.OptionsPanel.WorldMapOptionsHeader:SetSize(580, 50)
Forager.OptionsPanel.WorldMapOptionsHeader:SetFont("Fonts\\FRIZQT__.TTF", 13)
Forager.OptionsPanel.WorldMapOptionsHeader:SetJustifyH('LEFT')
Forager.OptionsPanel.WorldMapOptionsHeader:SetText('World map options')
--Forager.OptionsPanel.WorldMapOptionsHeader:SetTextColor(1,1,1,1)

-- show world map icons
Forager.OptionsPanel.WorldMapShowHerbsCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelWorldMapShowHerbsCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.WorldMapShowHerbsCheckBox:SetPoint('TOPLEFT', 20, -220)
ForagerOptionsPanelWorldMapShowHerbsCheckBoxText:SetText('|cffffffffHerbs|r')
Forager.OptionsPanel.WorldMapShowHerbsCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs = true
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs = false
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs = false
		end
	end
	Forager.Functions.Print('display herbs on world map: '..tostring(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs))
end)
Forager.OptionsPanel.WorldMapShowHerbsCheckBox.tooltip = 'Herbs.'

Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelWorldMapShowMetalAndStoneCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox:SetPoint('TOPLEFT', 20, -240)
ForagerOptionsPanelWorldMapShowMetalAndStoneCheckBoxText:SetText('|cffffffffMetal and Stone|r')
Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone = true
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone = false
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone = false
		end
	end
	Forager.Functions.Print('display metal & stone on world map: '..tostring(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone))
end)
Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox.tooltip = 'Metal and Stone.'

Forager.OptionsPanel.WorldMapShowChestsCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelWorldMapShowChestsCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.WorldMapShowChestsCheckBox:SetPoint('TOPLEFT', 20, -260)
ForagerOptionsPanelWorldMapShowChestsCheckBoxText:SetText('|cffffffffChests|r')
Forager.OptionsPanel.WorldMapShowChestsCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests = true
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests = false
		else
			ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests = false
		end
	end
	Forager.Functions.Print('display chests on world map: '..tostring(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests))
end)
Forager.OptionsPanel.WorldMapShowChestsCheckBox.tooltip = 'Chests.'

-- icon size world map
Forager.OptionsPanel.WorldMapIconSizeSlider = CreateFrame("Slider", "ForagerMinimapWorldMapIconSizeSlideBar", Forager.OptionsPanel, "OptionsSliderTemplate")
Forager.OptionsPanel.WorldMapIconSizeSlider:SetThumbTexture("Interface/Buttons/UI-SliderBar-Button-Horizontal")
Forager.OptionsPanel.WorldMapIconSizeSlider:SetSize(125, 20)
Forager.OptionsPanel.WorldMapIconSizeSlider:SetOrientation('HORIZONTAL')
Forager.OptionsPanel.WorldMapIconSizeSlider:SetPoint('TOPLEFT', 260, -220)
Forager.OptionsPanel.WorldMapIconSizeSlider:SetValue(8)
Forager.OptionsPanel.WorldMapIconSizeSlider:SetMinMaxValues(4, 20)
Forager.OptionsPanel.WorldMapIconSizeSlider:SetValueStep(1)
--get value when changed
Forager.OptionsPanel.WorldMapIconSizeSlider:SetScript('OnValueChanged', function(self) 
	getglobal(Forager.OptionsPanel.WorldMapIconSizeSlider:GetName() .. 'Text'):SetText(tostring('Icon Size: '..string.format('%.0f', tostring(self:GetValue()))))
	ForagerGlobalSettings[UnitGUID('player')].WorldMapIconSize = tonumber(self:GetValue())
end)
Forager.OptionsPanel.WorldMapIconSizeSlider.tooltipText = 'World Map Icon Size' --Creates a tooltip on mouseover.
getglobal(Forager.OptionsPanel.WorldMapIconSizeSlider:GetName()..'Low'):SetText('4')
getglobal(Forager.OptionsPanel.WorldMapIconSizeSlider:GetName()..'High'):SetText('20')
getglobal(Forager.OptionsPanel.WorldMapIconSizeSlider:GetName()..'Text'):SetText('Icon Size')











--minimap options
Forager.OptionsPanel.MiniMapOptionsHeader = Forager.OptionsPanel:CreateFontString('ForagerOptionsPanelMiniMapOptionsHeader', 'OVERLAY', 'GameFontNormal')
Forager.OptionsPanel.MiniMapOptionsHeader:SetPoint('TOPLEFT', 16, -300)
--Forager.OptionsPanel.MiniMapOptionsHeader:SetSize(580, 50)
Forager.OptionsPanel.MiniMapOptionsHeader:SetFont("Fonts\\FRIZQT__.TTF", 13)
Forager.OptionsPanel.MiniMapOptionsHeader:SetJustifyH('LEFT')
Forager.OptionsPanel.MiniMapOptionsHeader:SetText('Minimap options')
--Forager.OptionsPanel.MiniMapOptionsHeader:SetTextColor(1,1,1,1)


-- show minimap icons
Forager.OptionsPanel.MiniMapShowHerbsCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelMiniMapShowHerbsCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.MiniMapShowHerbsCheckBox:SetPoint('TOPLEFT', 20, -320)
ForagerOptionsPanelMiniMapShowHerbsCheckBoxText:SetText('|cffffffffHerbs|r')
Forager.OptionsPanel.MiniMapShowHerbsCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs = true
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs = false
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs = false
		end
	end
	Forager.Functions.Print('display herbs on minimap: '..tostring(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs))
end)
Forager.OptionsPanel.MiniMapShowHerbsCheckBox.tooltip = 'Herbs.'

Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelMiniMapShowMetalAndStoneCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox:SetPoint('TOPLEFT', 20, -340)
ForagerOptionsPanelMiniMapShowMetalAndStoneCheckBoxText:SetText('|cffffffffMetal and Stone|r')
Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone = true
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone = false
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone = false
		end
	end
	Forager.Functions.Print('display metal & stone on minimap: '..tostring(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone))
end)
Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox.tooltip = 'Metal and Stone.'

Forager.OptionsPanel.MiniMapShowChestsCheckBox = CreateFrame("CheckButton", 'ForagerOptionsPanelMiniMapShowChestsCheckBox', Forager.OptionsPanel, "ChatConfigCheckButtonTemplate")
Forager.OptionsPanel.MiniMapShowChestsCheckBox:SetPoint('TOPLEFT', 20, -360)
ForagerOptionsPanelMiniMapShowChestsCheckBoxText:SetText('|cffffffffChests|r')
Forager.OptionsPanel.MiniMapShowChestsCheckBox:SetScript('OnClick', function(self)
	if self:GetChecked() == true then
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests = true
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests = true
		end
	else
		if ForagerGlobalSettings[UnitGUID('player')] == nil then
			ForagerGlobalSettings[UnitGUID('player')] = {}
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests = false
		else
			ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests = false
		end
	end
	Forager.Functions.Print('display chests on minimap: '..tostring(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests))
end)
Forager.OptionsPanel.MiniMapShowChestsCheckBox.tooltip = 'Chests.'





-- icon size minimap
Forager.OptionsPanel.MinimapIconSizeSlider = CreateFrame("Slider", "ForagerMinimapMinimapIconSizeSlideBar", Forager.OptionsPanel, "OptionsSliderTemplate")
Forager.OptionsPanel.MinimapIconSizeSlider:SetThumbTexture("Interface/Buttons/UI-SliderBar-Button-Horizontal")
Forager.OptionsPanel.MinimapIconSizeSlider:SetSize(125, 20)
Forager.OptionsPanel.MinimapIconSizeSlider:SetOrientation('HORIZONTAL')
Forager.OptionsPanel.MinimapIconSizeSlider:SetPoint('TOPLEFT', 260, -320)
Forager.OptionsPanel.MinimapIconSizeSlider:SetValue(8)
Forager.OptionsPanel.MinimapIconSizeSlider:SetMinMaxValues(4, 20)
Forager.OptionsPanel.MinimapIconSizeSlider:SetValueStep(1)
--get value when changed
Forager.OptionsPanel.MinimapIconSizeSlider:SetScript('OnValueChanged', function(self) 
	getglobal(Forager.OptionsPanel.MinimapIconSizeSlider:GetName() .. 'Text'):SetText(tostring('Icon Size: '..string.format('%.0f', tostring(self:GetValue()))))
	ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize = tonumber(self:GetValue())
end)
Forager.OptionsPanel.MinimapIconSizeSlider.tooltipText = 'Minimap Icon Size' --Creates a tooltip on mouseover.
getglobal(Forager.OptionsPanel.MinimapIconSizeSlider:GetName()..'Low'):SetText('4')
getglobal(Forager.OptionsPanel.MinimapIconSizeSlider:GetName()..'High'):SetText('20')
getglobal(Forager.OptionsPanel.MinimapIconSizeSlider:GetName()..'Text'):SetText('Icon Size')

-- hide distance
Forager.OptionsPanel.HideMinimapPinDistanceSlider = CreateFrame("Slider", "ForagerHideMinimapPinDistanceSlideBar", Forager.OptionsPanel, "OptionsSliderTemplate")
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetThumbTexture("Interface/Buttons/UI-SliderBar-Button-Horizontal")
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetSize(125, 20)
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetOrientation('HORIZONTAL')
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetPoint('TOPLEFT', 440, -320)
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetValue(1.0)
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetMinMaxValues(0.5, 4.5)
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetValueStep(0.1)
--get value when changed
Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetScript('OnValueChanged', function(self) 
	getglobal(Forager.OptionsPanel.HideMinimapPinDistanceSlider:GetName() .. 'Text'):SetText(tostring('View Distance: '..string.format('%.2f', tostring(self:GetValue()))))
	ForagerGlobalSettings[UnitGUID('player')].HideMinimapPinDistance = tonumber(self:GetValue())
end)
Forager.OptionsPanel.HideMinimapPinDistanceSlider.tooltipText = 'Changes the distance that minimap pins will be hidden while exploring the world' --Creates a tooltip on mouseover.
getglobal(Forager.OptionsPanel.HideMinimapPinDistanceSlider:GetName()..'Low'):SetText('0.5')
getglobal(Forager.OptionsPanel.HideMinimapPinDistanceSlider:GetName()..'High'):SetText('4.5')
getglobal(Forager.OptionsPanel.HideMinimapPinDistanceSlider:GetName()..'Text'):SetText('View Distance')




Forager.OptionsPanel.HelpTitle = Forager.OptionsPanel:CreateFontString("ForagerOptionsPanelTitleHelp", "OVERLAY", "GameFontNormal")
Forager.OptionsPanel.HelpTitle:SetPoint("TOPLEFT", 16, -400)
Forager.OptionsPanel.HelpTitle:SetFont("Fonts\\FRIZQT__.TTF", 13)
Forager.OptionsPanel.HelpTitle:SetText("Help & Commands")
--about
Forager.OptionsPanel.Help = Forager.OptionsPanel:CreateFontString('ForagerOptionsPanelHelp', 'OVERLAY', 'GameFontNormal')
Forager.OptionsPanel.Help:SetPoint('TOPLEFT', 16, -420)
Forager.OptionsPanel.Help:SetSize(580, 50)
Forager.OptionsPanel.Help:SetFont("Fonts\\FRIZQT__.TTF", 11)
Forager.OptionsPanel.Help:SetJustifyH('LEFT')
Forager.OptionsPanel.Help:SetText([[
'/forager purge' opens a dialog where you can delete the local database
'/forager capture' opens a small window and allows you to update the addon if any gathering spells change
'/forager list-spells' will prin the currently tracked spells used for gathering
'/forager remove-spell' opens a dialog where a spell can be removed from the tracking list
]])
Forager.OptionsPanel.Help:SetTextColor(1,1,1,1)




--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- CAPTURE FRAME
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


Forager.CaptureFrame = CreateFrame('FRAME', 'ForagerCaptureFrame', UIParent)
Forager.CaptureFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
												edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
												tile = true, tileSize = 16, edgeSize = 16, 
												insets = { left = 4, right = 4, top = 4, bottom = 4 }});
Forager.CaptureFrame:SetBackdropColor(0, 0, 0, 1)
Forager.CaptureFrame:SetSize(150, 70)
Forager.CaptureFrame:SetPoint('CENTER', 0, 0)
Forager.CaptureFrame:IsClampedToScreen(true)

Forager.CaptureFrame.Header = Forager.CaptureFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
Forager.CaptureFrame.Header:SetText('Capture Node')
Forager.CaptureFrame.Header:SetPoint('TOP', 0, -8)

-- Forager.CaptureFrame.SpellCastIDText = Forager.CaptureFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
-- Forager.CaptureFrame.SpellCastIDText:SetText('Collect the node to check spell ID')
-- Forager.CaptureFrame.SpellCastIDText:SetPoint('TOPLEFT', 6, -28)

Forager.CaptureFrame.SpellIDExist = Forager.CaptureFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
Forager.CaptureFrame.SpellIDExist:SetText('')
Forager.CaptureFrame.SpellIDExist:SetPoint('TOP', 0, -28)

Forager.CaptureFrame.AddButton = CreateFrame("BUTTON", "ForagerCaptureFrameAddButton", Forager.CaptureFrame, "UIPanelButtonTemplate")
Forager.CaptureFrame.AddButton:SetText('|cFFFFFFFFCapture|r')
Forager.CaptureFrame.AddButton:SetPoint("TOP", 0, -43)
Forager.CaptureFrame.AddButton:SetEnabled(false)
Forager.CaptureFrame.AddButton:SetSize(100, 22)
Forager.CaptureFrame.AddButton:SetScript('OnClick', function() 
	local spell = select(1, GetSpellInfo(Forager.CaptureSpellID))
	StaticPopup_Show ("Forager_AddAbility", spell)
end)
Forager.CaptureFrame:Hide()




--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- WORLD MAP TOOLTIP
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Forager.NodeToolTip = CreateFrame('FRAME', 'ForagerNodeToolTip', UIParent)
Forager.NodeToolTip:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
												edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
												tile = true, tileSize = 16, edgeSize = 16, 
												insets = { left = 4, right = 4, top = 4, bottom = 4 }});
Forager.NodeToolTip:SetBackdropColor(0, 0, 0, 1)
Forager.NodeToolTip:SetSize(150, 40)
Forager.NodeToolTip:SetFrameLevel(4)
Forager.NodeToolTip:SetFrameStrata('FULLSCREEN_DIALOG')
ForagerNodeToolTip:Hide()

Forager.NodeToolTip.Text = Forager.NodeToolTip:CreateFontString(nil, "OVERLAY", "GameFontNormal")
Forager.NodeToolTip.Text:SetPoint('CENTER', 0, 0)
--Forager.NodeToolTip.Text:SetTextColor(1,1,1,1)



--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- FUNCTIONS
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--[==[
- CREATE A NEW FRAME
@parent - parent frame for this frame
@name - global name for new frame
@x, y - the x, y coords for the frame using bottomleft anchor
@w - width of new frame
@h - height of new frame
@r, g, b, a - the rgbs colour settings for new frame
]==]--
function Forager.DrawFrame(parent, name, x, y, w, h, r, g, b, a)
	local f = CreateFrame('FRAME', name, parent)
	f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
					edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
					tile = true, tileSize = 16, edgeSize = 16, 
					insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	f:SetBackdropColor(r, g, b, a)
	f:SetSize(w, h)
	f:SetPoint('BOTTOMLEFT', x, y)
end

-- print text message to chat window
function Forager.Functions.Print(msg, override)
	
	--local c = Forager.ClassColours[UnitClass('player')].fs
	--msg = tostring('|c'..c..'Forager:|r '.msg)
	
	if ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo == true or override == true then
	
		print('|cff00FF96Forager: |r'..msg)
		
	end
end

--------------------------------------------------------------------------------------------------
-- should be consolidated into 1 function

--[==[
- SET A FRAME AS AN ICON FOR WORLD MAP
@frame - frame to convert
@x, y - the x, y coords for the frame using bottomleft anchor
@w - width of new frame
@h - height of new frame
@level - frame draw level, see wow api
@strata - frame strata level, see wow api
@icon - fileID for icon texture
]==]--
function Forager.DrawMapNodeIcon(f, x, y, w, h, level, strata, icon)

	f:SetPoint("TOPLEFT", x, (y * -1)) -- convert Y axis to negative number to draw pin from top left anchor
	f:SetSize(w, h)
	f:SetFrameLevel(level)
	f:SetFrameStrata(strata)
	f:EnableKeyboard(true)
	f:SetPropagateKeyboardInput(true)
	
	--f:SetScript('OnLeave', Forager.HideForagerToolTip)
	
	f.texture = f:CreateTexture(nil, 'ARTWORK')
	f.texture:SetAllPoints(f)
	f.texture:SetTexture(icon)
end

--[==[
- SET A FRAME AS AN ICON FOR MINIMAP
@frame - frame to convert
@x, y - the x, y coords for the frame using bottomleft anchor
@w - width of new frame
@h - height of new frame
@level - frame draw level, see wow api
@strata - frame strata level, see wow api
@icon - fileID for icon texture
]==]--
function Forager.DrawMapNodeIconMinimap(f, x, y, w, h, level, strata, icon)

	f:SetPoint("CENTER", x, y)
	f:SetSize(w, h)
	f:SetFrameLevel(level)
	f:SetFrameStrata(strata)
	f:EnableKeyboard(true)
	f:SetPropagateKeyboardInput(true)
	
	f.texture = f:CreateTexture(nil, 'ARTWORK')
	f.texture:SetAllPoints(f)
	f.texture:SetTexture(icon)
end

--------------------------------------------------------------------------------------------------

--print event args to chat window
function Forager.Functions.GetArgs(...)
	for i=1, select("#", ...) do
		arg = select(i, ...)
		print(i.." "..tostring(arg))
	end
end

function Forager.GetFormattedDate(dateObj)
	local d, mo, h, mi, s = dateObj.day, dateObj.month, dateObj.hour, dateObj.min, dateObj.sec
	if string.len(dateObj.day) == 1 then
		d = tostring('0'..dateObj.day)
	end
	if string.len(dateObj.month) == 1 then
		mo = tostring('0'..dateObj.month)
	end
	if string.len(dateObj.min) == 1 then
		mi = tostring('0'..dateObj.min)
	end
	if string.len(dateObj.hour) == 1 then
		h = tostring('0'..dateObj.hour)
	end
	if string.len(dateObj.month) == 1 then
		m = tostring('0'..dateObj.month)
	end
	if string.len(dateObj.sec) == 1 then
		s = tostring('0'..dateObj.sec)
	end
	
	return tostring(d..'/'..mo..'/'..dateObj.year..' '..h..':'..mi..':'..s)
end

--find the item id from the hyperlink string
function Forager.ItemIDFromLootString(str)
	local s = string.find(str, 'Hitem:')
	local t = string.sub(str, s + 6)
	local f = string.find(t, ':')
	local n = string.sub(t, 1, f - 1)
	--print(n)
	return n
end

-- attempts to extract the loot count from the loot message OLD FUNCTION FROM FIRST VERSION WITH NODE LOOT DATA
function Forager.ItemLootCountFromString(str)
	local s = string.find(str, '|r')
	--print('|r is at', s, 'length is', string.len(str))
	local c = string.sub(str, s + 1)
	--print(c)
	if string.find(c, 'x') then
		return tonumber(string.sub(str, s + 3, string.len(str)-1)) --ignore full stop '.' at end
	else
		return 1 -- return 1 if no 'x?' in string
	end
end

-- dirty hack to hide all minimap pins
function Forager.HideMinimapPins()
	for i = 1, 100 do  -- using 100 as max node displayed - try to improve this idea/system as its a dirty hack
		if Forager.MapNodeCacheMinimap[i] then
			Forager.MapNodeCacheMinimap[i]:Hide()
		end
	end
end

-- classic doesnt provide the item sub type via GetItemInfo() so this sets a custom subtype to determine if node is displayed based on tracking type
function Forager.GetItemTypeFromTarget(target)
	local t = nil
	for _, k in pairs(Forager.Herbs) do
		if k == target then
			return 'Herb'
		end
	end
	for _, j in pairs(Forager.Ore) do
		if j == target then
			return 'Metal & Stone'
		end
	end
	if t == nil then
		if string.find(string.lower(target), 'chest') then
			return 'Chest'
		end
	end
end

--check node coords with current coords, if within double spell range then assume same node (not often to find 2 nodes together and if player does then loot info will be combined)
function Forager.IsNodeInRange(myPosX, myPosY, nodePosX, nodePosY)
	local distance = (((myPosX - nodePosX)^2.0) + ((myPosY - nodePosY)^2.0)) ^ 0.5
	if distance < Forager.GatherSpellRange then
		--print(distance * 140)
		return true
	else
		return false
	end
	
end

-- check if node is in range for minimap display CAN BE IMPROVED TO REMOVE ALL THE ELSEIF CHECKS
function Forager.IsNodeInRangeMinimap(myPosX, myPosY, nodePosX, nodePosY)
	--local distance = (((((myPosX * -1.0) - (nodePosX * -1.0))^2.0) + (((myPosY * -1.0) - (nodePosY * -1.0))^2.0)) ^ 0.5)
	local distance = (((myPosX - nodePosX)^2.0) + ((myPosY - nodePosY)^2.0)) ^ 0.5
	distance = distance * 140
	local zoom = Minimap:GetZoom()
	if zoom == 5 then
		if distance < 1.8 then
			return true
		else
			return false
		end
	elseif zoom == 4 then
		if distance < 2.4 then
			return true
		else
			return false
		end
	elseif zoom == 3 then
		if distance < 3.0 then
			return true
		else
			return false
		end
	elseif zoom == 2 then
		if distance < 3.8 then
			return true
		else
			return false
		end
	elseif zoom == 1 then
		if distance < 5.0 then
			return true
		else
			return false
		end
	elseif zoom == 0 then
		if distance < 6.0 then
			return true
		else
			return false
		end
	end
end

-- this function will loop the addon database and update/create pins and display them on the world map for the zone that is currently displayed
function Forager.UpdateMapPins()

	local x, y = GetCursorPosition()
	local s = UIParent:GetEffectiveScale()
	Forager.NodeToolTip:ClearAllPoints()
		
	Forager.IsZoneDisplayedInZoneTable = false -- check if zone exists in addon db

	--get the current map zone displayed in world map and check if its a zone in the db
	for k, zone in ipairs(Forager.MapZoneTable) do
		if zone == C_Map.GetMapInfo(WorldMapFrame:GetMapID()).name then
			Forager.IsZoneDisplayedInZoneTable = true
			Forager.CurrentMapID = WorldMapFrame:GetMapID()
		end
	end
	
	for k, p in ipairs(Forager.MapNodeCache) do 
		p:Hide()
	end
	
	--if zone exist in zone db and node db has entries for zone
	if Forager.IsZoneDisplayedInZoneTable == true and ForagerGlobalSettings.NodesLootedDatabase[C_Map.GetMapInfo(Forager.CurrentMapID).name] ~= nil then

		local MapWidth = WorldMapFrame.ScrollContainer.Child:GetWidth()
		local MapHeight = WorldMapFrame.ScrollContainer.Child:GetHeight()
			
		for k, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[C_Map.GetMapInfo(Forager.CurrentMapID).name]) do
						
			local icon = nil
			if node.SubType == 'Chest' then
				icon = 132594
			else
				icon = GetItemIcon(node.Loot)
			end
			local showNode = false
			
				if ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests == true and node.SubType == 'Chest' then
					showNode = true
				elseif ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs == true and node.SubType == 'Herb' then
					showNode = true
				elseif ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone == true and node.SubType == 'Metal & Stone' then
					showNode = true
				end
				
				if showNode == true then

					if Forager.MapNodeCache[k] == nil then
						--no pin in cache, create frame for pin and pass to drawing function
						local Pin = CreateFrame('FRAME', 'ForagerMapNodePin'..k, WorldMapFrame.ScrollContainer.Child)
						Forager.DrawMapNodeIcon(Pin, (node.PosX * MapWidth), (node.PosY * MapHeight), tonumber(ForagerGlobalSettings[UnitGUID('player')].WorldMapIconSize), tonumber(ForagerGlobalSettings[UnitGUID('player')].WorldMapIconSize), 4, 'FULLSCREEN_DIALOG', icon)
						Forager.MapNodeCache[k] = Pin
						--add onenter script
						-- Forager.MapNodeCache[k]:SetScript('OnEnter', function()
							-- Forager.NodeToolTip:SetPoint("BOTTOMLEFT", (x/s), (y/s))
							-- Forager.NodeToolTip.Text:SetText(tostring(node.Target..'\n '..node.VisitDate.day .. '/' .. node.VisitDate.month .. '/' .. node.VisitDate.year .. " - " .. node.VisitDate.hour .. ':' .. node.VisitDate.min .. ':' .. node.VisitDate.sec)) 
						-- end)
						-- Forager.MapNodeCache[k]:SetScript('OnLeave', function()
							-- Forager.NodeToolTip:Hide()
						-- end)					
					else
						-- re use existing frame for pin
						Forager.MapNodeCache[k]:SetPoint("TOPLEFT", (node.PosX * MapWidth), ((node.PosY * MapHeight) * -1))
						Forager.MapNodeCache[k].texture:SetTexture(icon)
						Forager.MapNodeCache[k]:Show()
						-- Forager.MapNodeCache[k]:SetScript('OnEnter', function()
							-- Forager.NodeToolTip:SetPoint("BOTTOMLEFT", (x/s), (y/s))
							-- Forager.NodeToolTip.Text:SetText(tostring(node.Target..'\n '..node.VisitDate.day .. '/' .. node.VisitDate.month .. '/' .. node.VisitDate.year .. " - " .. node.VisitDate.hour .. ':' .. node.VisitDate.min .. ':' .. node.VisitDate.sec)) 
						-- end)
						-- Forager.MapNodeCache[k]:SetScript('OnLeave', function()
							-- Forager.NodeToolTip:Hide()
						-- end)
					end
				end
			--end
		end
	end
end


-- this needs some restructuring to improve performance ?
function Forager.UpdateMapPinsMinimap()

	local x, y = GetCursorPosition()
	local s = UIParent:GetEffectiveScale()
	Forager.NodeToolTip:ClearAllPoints()

	local zoom = Minimap:GetZoom()

	local distanceMod = Forager.MinimapScales[zoom]

	--grab current position
	Forager.CurrentMapID = C_Map.GetBestMapForUnit('player')
	if not Forager.CurrentMapID then
		return
	else
	Forager.CurrentMapPosition = C_Map.GetPlayerMapPosition(Forager.CurrentMapID, 'player')
	Forager.CurrentMapName = C_Map.GetMapInfo(Forager.CurrentMapID).name
	
	if Forager.CurrentMapPosition ~= nil then

		if ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] ~= nil then
			for k, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName]) do
				
			local showNode = false
			
				if ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests == true and node.SubType == 'Chest' then
					showNode = true
				elseif ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs == true and node.SubType == 'Herb' then
					showNode = true
				elseif ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone == true and node.SubType == 'Metal & Stone' then
					showNode = true
				end
				
				if showNode == true then

						if Forager.IsNodeInRangeMinimap(Forager.CurrentMapPosition.x, Forager.CurrentMapPosition.y, node.PosX, node.PosY) then
							
							local xDiff = (Forager.CurrentMapPosition.x * 140.0) - (node.PosX * 140.0)
							local yDiff = (Forager.CurrentMapPosition.y * 140.0) - (node.PosY * 140.0)
							
							local distance = ((xDiff^2.0) + (yDiff^2.0))^0.5
							
							local icon = nil
							if node.SubType == 'Chest' then
								icon = 132594
							else
								icon = GetItemIcon(node.Loot)
							end
							
							-- cretse first node
							if Forager.MapNodeCacheMinimap[k] == nil then
								local PinMinimap = CreateFrame('FRAME', 'ForagerMapNodePinMinimap'..k, Minimap)
								Forager.DrawMapNodeIconMinimap(PinMinimap, (distanceMod * (xDiff * -1.0)), (distanceMod * yDiff), tonumber(ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize), tonumber(ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize), 4, 'FULLSCREEN_DIALOG', icon)
								Forager.MapNodeCacheMinimap[k] = PinMinimap
								Forager.MapNodeCacheMinimap[k]:SetScript('OnEnter', function()
									Forager.NodeToolTip:SetPoint("BOTTOMLEFT", (x/s)-180, (y/s))
									Forager.NodeToolTip.Text:SetText(tostring(node.Target..'\n '..Forager.GetFormattedDate(node.VisitDate)))
									Forager.NodeToolTip:Show()
								end)
								Forager.MapNodeCacheMinimap[k]:SetScript('OnLeave', function()
									Forager.NodeToolTip:Hide()
								end)
							else
								Forager.MapNodeCacheMinimap[k]:SetPoint('CENTER', (distanceMod * 1.5) * (xDiff * -1.0) , (distanceMod * 1.0) * yDiff )
								Forager.MapNodeCacheMinimap[k].texture:SetTexture(icon)
								Forager.MapNodeCacheMinimap[k]:SetScript('OnEnter', function()
									Forager.NodeToolTip:SetPoint("BOTTOMLEFT", (x/s)-180, (y/s))
									Forager.NodeToolTip.Text:SetText(tostring(node.Target..'\n '..Forager.GetFormattedDate(node.VisitDate)))
									Forager.NodeToolTip:Show()
								end)
								Forager.MapNodeCacheMinimap[k]:SetScript('OnLeave', function()
									Forager.NodeToolTip:Hide()
								end)								
								Forager.MapNodeCacheMinimap[k]:Show()
								Forager.MapNodeCacheMinimap[k]:SetSize(tonumber(ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize), tonumber(ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize))
							end

							if distance < tonumber(ForagerGlobalSettings[UnitGUID('player')].HideMinimapPinDistance) then
								Forager.MapNodeCacheMinimap[k]:Hide()
							end
							
						else --?
							if Forager.MapNodeCacheMinimap[k] ~= nil then
								Forager.MapNodeCacheMinimap[k]:Hide()
							end

						end

				else
					Forager.HideMinimapPins()
				end
			end
		end
	end
	end
end

-- takes the string message and splits it then adds to local db
function Forager.Functions.ParseSentData(msg, sender)
	
	-- make sure we have a variable for the db
	if ForagerGlobalSettings == nil then
		ForagerGlobalSettings = { AddonName = 'Forager', NodesLootedDatabase = {} }
	end
	
	-- local table to hold the seperated msg 
	local data = {}
	local i = 1
	for d in string.gmatch(msg, '[^:]+') do
		data[i] = d
		i = i + 1
		--print(i, data[i])
	end
	
	-- ignore data that we sent
	if sender ~= UnitName('player') then
		if data[1] == 'newdata' then
			--print('message prefix',data[1],'map name', data[2], 'map id', data[3], 'X', data[4], 'Y', data[5], 'loot id', data[6])
			local updated = false
			if ForagerGlobalSettings.NodesLootedDatabase[data[2]] ~= nil then
				for k, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[data[2]]) do
					--check if we have been at this node before
					if Forager.IsNodeInRange(data[4], data[5], node.PosX, node.PosY) then
						Forager.Functions.Print('You\'ve been told about loot you\'ve alrealy found!')
						node.VisitDate = date('*t')
						--if node has been updated change bool check
						updated = true
					end
				end
			end

		
			--if node wasn't updated then we add a new entry
			if updated == false then
				Forager.Functions.Print('Friendly message about some treasure!')
				local node = { Loot = data[6], MapName = data[2], MapID = data[3], PosX = data[4], PosY = data[5], VisitDate = date('*t'), Target = data[7], SubType = data[8] }
				table.insert(ForagerGlobalSettings.NodesLootedDatabase[data[2]], node)
			end
		end
	end
		

end

function Forager.Functions.FoundChest()

	Forager.CurrentMapID = C_Map.GetBestMapForUnit('player')
	if not Forager.CurrentMapID then
		return
	else
		Forager.CurrentMapPosition = C_Map.GetPlayerMapPosition(Forager.CurrentMapID, 'player')
		Forager.CurrentMapName = C_Map.GetMapInfo(Forager.CurrentMapID).name
		
		--check if a table exist for this map zone
		if ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] == nil then
			ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] = {}
		end
		
		--variables just for ease of reading
		local LootItemID = Forager.ItemIDFromLootString(loot)
		--local LootItemCount = Forager.ItemLootCountFromString(loot)
		local subType
		
		--reset the bool check
		Forager.NodeUpdated = false

		--check we have coords
		if Forager.CurrentMapPosition ~= nil then
			--search db and add or updated
			if ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] ~= nil then
				for k, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName]) do
					--check if we have been at this node before
					if Forager.IsNodeInRange(Forager.CurrentMapPosition.x, Forager.CurrentMapPosition.y, node.PosX, node.PosY) then
						Forager.Functions.Print('found node in database, updating visit time')
						node.VisitDate = date('*t')
						--if node has been updated change bool check
						Forager.NodeUpdated = true
					end
				end
			end
			
			--if node wasn't updated then we add a new entry
			if Forager.NodeUpdated == false then
				Forager.Functions.Print('new node found, adding location to database')
		
				local node = { Target = Forager.SpellTarget, SubType = 'Chest', Loot = 'unknown', MapName = Forager.CurrentMapName, MapID = Forager.CurrentMapID, PosX = Forager.CurrentMapPosition.x, PosY = Forager.CurrentMapPosition.y, VisitDate = date('*t') }
				table.insert(ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName], node)
			end
		end
		if ForagerGlobalSettings[UnitGUID('player')].GuildShare then
			local dataString = tostring('newdata'..':'..Forager.CurrentMapName..':'..Forager.CurrentMapID..':'..Forager.CurrentMapPosition.x..':'..Forager.CurrentMapPosition.y..':'..LootItemID..':'..Forager.SpellTarget..':'..Forager.GetItemTypeFromTarget(Forager.SpellTarget))
			C_ChatInfo.SendAddonMessage('forager', dataString, 'GUILD')
		end
		Forager.ChestLooted = false
	end
end


function Forager.Functions.FoundNode(loot)

	Forager.CurrentMapID = C_Map.GetBestMapForUnit('player')
	if not Forager.CurrentMapID then
		return
	else
		Forager.CurrentMapPosition = C_Map.GetPlayerMapPosition(Forager.CurrentMapID, 'player')
		Forager.CurrentMapName = C_Map.GetMapInfo(Forager.CurrentMapID).name
		
		--check if a table exist for this map zone
		if ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] == nil then
			ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] = {}
		end
		
		--variables just for ease of reading
		local LootItemID = Forager.ItemIDFromLootString(loot)
		--local LootItemCount = Forager.ItemLootCountFromString(loot)
		local subType
		
		--reset the bool check
		Forager.NodeUpdated = false

		--check we have coords
		if Forager.CurrentMapPosition ~= nil then
			--search db and add or updated
			if ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName] ~= nil then
				for k, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName]) do
					--check if we have been at this node before
					if Forager.IsNodeInRange(Forager.CurrentMapPosition.x, Forager.CurrentMapPosition.y, node.PosX, node.PosY) then
						Forager.Functions.Print('found node in database, updating visit time')
						node.VisitDate = date('*t')
						--if node has been updated change bool check
						Forager.NodeUpdated = true
					end
				end
			end
			
			--if node wasn't updated then we add a new entry
			if Forager.NodeUpdated == false then
				Forager.Functions.Print('new node found, adding location to database')
				
				if string.find(tostring(GetAddOnMetadata('Forager', 'Version')), '8.') then
					subType = select(7, GetItemInfo(LootItemID))
				else
					subType = Forager.GetItemTypeFromTarget(Forager.SpellTarget)
				end
				
				local node = { Target = Forager.SpellTarget, SubType = subType, Loot = LootItemID, MapName = Forager.CurrentMapName, MapID = Forager.CurrentMapID, PosX = Forager.CurrentMapPosition.x, PosY = Forager.CurrentMapPosition.y, VisitDate = date('*t') }
				table.insert(ForagerGlobalSettings.NodesLootedDatabase[Forager.CurrentMapName], node)
			end
		end
		
		-- send database
		if ForagerGlobalSettings[UnitGUID('player')].GuildShare then
			if string.find(tostring(GetAddOnMetadata('Forager', 'Version')), '8.') then
				subType = select(7, GetItemInfo(LootItemID))
			else
				subType = Forager.GetItemTypeFromTarget(Forager.SpellTarget)
			end
			local dataString = tostring('newdata'..':'..Forager.CurrentMapName..':'..Forager.CurrentMapID..':'..Forager.CurrentMapPosition.x..':'..Forager.CurrentMapPosition.y..':'..LootItemID..':'..Forager.SpellTarget..':'..subType)
			C_ChatInfo.SendAddonMessage('forager', dataString, 'GUILD')
		end
	end
	
end

-- gets called 2 seconds after addon loaded, enables variables to be read without issue
function Forager.Functions.OnLoad()

	--update db
	if ForagerGlobalSettings.AddonVersion == GetAddOnMetadata('Forager', 'Version') then
		Forager.Functions.Print('using latest version')
	else
		Forager.Functions.Print('updating addon...')
		ForagerGlobalSettings.SpellIDs = { 2575, 2576, 3564, 10248, 29354, 50310, 74517, 102161, 158754, 195122, 265837, 2366, 2368, 3570, 11993, 28695, 50300, 74519, 110413, 158745, 195114, 265819, 3365 }
		for k, zone in pairs(Forager.MapZoneTable) do
			if ForagerGlobalSettings.NodesLootedDatabase[zone] then
				for j, node in ipairs(ForagerGlobalSettings.NodesLootedDatabase[zone]) do
					if node.SubType == 'Ore' then
						node.SubType = 'Metal & Stone'
					end
				end
			end
		end
		ForagerGlobalSettings.AddonVersion = GetAddOnMetadata('Forager', 'Version')
		Forager.Functions.Print('updated to version '..GetAddOnMetadata('Forager', 'Version'))
	end
		
		
	
	if ForagerGlobalSettings[UnitGUID('player')] == nil then
		ForagerGlobalSettings[UnitGUID('player')] = { GuildShare = false, ShowMinimapIcons = false, MiniMapShowHerbs = true, MiniMapShowMetalAndStone = true, MiniMapShowChests = true, WorldMapShowHerbs = true, WorldMapShowMetalAndStone = true, WorldMapShowChests = true, WorldMapIconSize = 8.0, MinimapIconSize = 8.0, GameMode = nil, HideMinimapPinDistance = 1.0, PrintForagerInfo = true }
		Forager.OptionsPanel.GuildShareCheckBox:SetChecked(false)
		
		Forager.OptionsPanel.WorldMapShowHerbsCheckBox:SetChecked(true)
		Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox:SetChecked(true)
		Forager.OptionsPanel.WorldMapShowChestsCheckBox:SetChecked(true)
		
		Forager.OptionsPanel.MiniMapShowHerbsCheckBox:SetChecked(true)
		Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox:SetChecked(true)
		Forager.OptionsPanel.MiniMapShowChestsCheckBox:SetChecked(true)
		
		Forager.OptionsPanel.PrintForagerInfoCheckBox:SetChecked(true)
		
		Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetValue(1.0)
		Forager.OptionsPanel.MinimapIconSizeSlider:SetValue(8.0)
		
		Forager.OptionsPanel.WorldMapIconSizeSlider:SetValue(8.0)
		Forager.Functions.Print(tostring('created settings for '..UnitName('player')))
	else
		
		--add new settings to existing player
		ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs = true
		ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone = true
		ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests = true
		
		ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs = true
		ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone = true
		ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests = true
		-- end
	
		Forager.OptionsPanel.GuildShareCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].GuildShare)

		Forager.OptionsPanel.WorldMapShowHerbsCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowHerbs)
		Forager.OptionsPanel.WorldMapShowMetalAndStoneCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowMetalAndStone)
		Forager.OptionsPanel.WorldMapShowChestsCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].WorldMapShowChests)
		
		Forager.OptionsPanel.MiniMapShowHerbsCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowHerbs)
		Forager.OptionsPanel.MiniMapShowMetalAndStoneCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowMetalAndStone)
		Forager.OptionsPanel.MiniMapShowChestsCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].MiniMapShowChests)
		
		Forager.OptionsPanel.PrintForagerInfoCheckBox:SetChecked(ForagerGlobalSettings[UnitGUID('player')].PrintForagerInfo)
		
		Forager.OptionsPanel.HideMinimapPinDistanceSlider:SetValue(tonumber(ForagerGlobalSettings[UnitGUID('player')].HideMinimapPinDistance))
		Forager.OptionsPanel.MinimapIconSizeSlider:SetValue(tonumber(ForagerGlobalSettings[UnitGUID('player')].MinimapIconSize))
		
		Forager.OptionsPanel.WorldMapIconSizeSlider:SetValue(tonumber(ForagerGlobalSettings[UnitGUID('player')].WorldMapIconSize))
		print('|cff00FF96Forager: |r'..tostring('loaded settings for '..UnitName('player')))
	end
	
	-- experimental system, makes minimap less smooth but maybe is better on hardware performance?
	-- Forager.RefreshTicker = C_Timer.NewTicker(tonumber(0.1), function() 
	
		-- if ForagerGlobalSettings ~= nil and ForagerGlobalSettings[UnitGUID('player')] ~= nil then
		
			-- if WorldMapFrame:IsVisible() and ForagerGlobalSettings[UnitGUID('player')].ShowWorldmapIcons then
				-- Forager.UpdateMapPins()
			-- end
			
			-- if ForagerGlobalSettings[UnitGUID('player')].ShowMinimapIcons then
				-- Forager.UpdateMapPinsMinimap()
			-- else
				-- Forager.HideMinimapPins()
			-- end
			
		-- end

	-- end)
	
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- EVENTS
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function Forager.OnEvent(self, event, ...)

	--establish saved variables and inform plauyer
	if event == 'ADDON_LOADED' and string.lower(select(1, ...)) == 'forager' then
		if ForagerGlobalSettings == nil then
			ForagerGlobalSettings = { AddonName = 'Forager', NodesLootedDatabase = {} }
			ForagerGlobalSettings.SpellIDs = { 2575, 2576, 3564, 10248, 29354, 50310, 74517, 102161, 158754, 195122, 265837, 2366, 2368, 3570, 11993, 28695, 50300, 74519, 110413, 158745, 195114, 265819, 3365 }
		end
		
		if C_ChatInfo.RegisterAddonMessagePrefix("forager") then			
			print('|cff00FF96Forager: |r'..'addon channel prefix registration successful!')
		else
			print('|cff00FF96Forager: |r'..'addon channel prefix registration failed!')		
		end
		
		C_Timer.After( 2, function()  Forager.Functions.OnLoad() end)
			
	end

	-- if the player starts to gather but gets interupted then we reset bool checks otherwise mob loot would be collected as node loot
	if event == 'PLAYER_REGEN_DISABLED' then
		Forager.LastLootSpell = nil
		Forager.IsForagingSpellCast = false
		Forager.ForagingTarget = nil
		Forager.SpellTarget = 'none'
	end
		

	if event == 'UNIT_SPELLCAST_SENT' then

		local t = select(2, ...)
		local spellID = select(4, ...)
		local foraged = false
		
		if Forager.CaptureFrame:IsVisible() then
			local add = true
			--Forager.CaptureFrame.SpellCastIDText:SetText(tostring(Forager.CaptureFrame.SpellCastIDText:GetText()..spellID))
			for k, spell in pairs(ForagerGlobalSettings.SpellIDs) do
				if spell == spellID then
					add = false
				end
			end
			if add == true then
				Forager.CaptureFrame.SpellIDExist:SetText('New spell detected')
				Forager.CaptureFrame.AddButton:SetEnabled(true)
				Forager.CaptureSpellID = spellID
			else
				Forager.CaptureFrame:Hide()
			end
		end

		--Forager.Functions.GetArgs(...)
		-- checks if the spell cast is in the table for game version spells
		for k, s in pairs(ForagerGlobalSettings.SpellIDs) do
			if s == spellID then
				Forager.LastLootSpell = spellID
				Forager.IsForagingSpellCast = true
				Forager.SpellTarget = t
				foraged = true
			end
		end
		if foraged == false then
			Forager.LastLootSpell = nil
			Forager.IsForagingSpellCast = false
			Forager.SpellTarget = 'none'
		end
		if spellID == 3365 then
			Forager.ChestLooted = true
		end
	end
	
	-- if the player gets loot and the last cast was a herb/mining spell then we found a node
	if event == 'CHAT_MSG_LOOT' and Forager.IsForagingSpellCast == true then		
		local loot = select(1, ...)
		if Forager.ChestLooted == true then
			Forager.Functions.FoundChest()
			Forager.ChestLooted = false
		else
			Forager.Functions.FoundNode(loot)
		end
			
		
	end
	
	--guild share option when message is sent this reads it 
	if event == 'CHAT_MSG_ADDON' and select(1, ...) == 'forager' then
		local msg = select(2, ...)
		local sender = select(5, ...)
		local prefix = select(1, ...)
		
		if prefix == 'newdata' then
			Forager.Functions.ParseSentData(msg, sender)
		end
	end
	

end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- UPDATE
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function Forager.OnUpdate()


	if ForagerGlobalSettings ~= nil and ForagerGlobalSettings[UnitGUID('player')] ~= nil then
	
		if WorldMapFrame:IsVisible() and ForagerGlobalSettings[UnitGUID('player')].ShowWorldmapIcons then
			Forager.UpdateMapPins()
		end
		
		if ForagerGlobalSettings[UnitGUID('player')].ShowMinimapIcons then
			Forager.UpdateMapPinsMinimap()
		else
			Forager.HideMinimapPins()
		end
		
	end
	


end




--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- HOOK SECURE
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--MiniMapTrackingFrame:SetScript('OnClick', function() print('tracking click') end)


--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- SET SCRIPTS
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Forager.MainFrame:SetScript('OnEvent', Forager.OnEvent)
Forager.MainFrame:SetScript('OnUpdate', Forager.OnUpdate)


